import { createObserver } from "./create-observer";
import debounce from "debounce";
import { getCSSValues } from "./get-css-values";
import LayoutWorker from "web-worker:./layout-worker";

function createLayout(target, onUpdate) {
  const elements = document.querySelectorAll(target);

  elements.forEach((el) => {
    const layoutWorker = new LayoutWorker();

    layoutWorker.onmessage = (e) => {
      onUpdate(e.data, el);
    };

    let first = true;
    let canRunDebounce = false;

    const run = debounce(
      (elementWidth, elementHeight, contentRects, config) => {
        layoutWorker.postMessage({
          elementWidth,
          elementHeight,
          contentRects,
          viewboxMode: !!config.layoutWidth && !!config.layoutHeight,
          ...config,
        });
      },
      256
    );

    createObserver(el, (elementWidth, elementHeight, contentRects) => {
      const cssValues = getCSSValues(el);
      const config = {
        layoutWidth: cssValues["--layout-width"],
        layoutHeight: cssValues["--layout-height"],
        minContentGap: cssValues["--min-content-gap"],
        minPatternGap: cssValues["--min-pattern-gap"],
        cellDimensions: {
          min: cssValues["--min-cell-size"],
          max: cssValues["--max-cell-size"],
        },
        seed: cssValues["--seed"],
        fillIterations: cssValues["--fill-iterations"],
      };

      if (first) {
        layoutWorker.postMessage({
          elementWidth,
          elementHeight,
          contentRects,
          viewboxMode: !!config.layoutWidth && !!config.layoutHeight,
          ...config,
        });

        first = false;
        setTimeout(() => {
          canRunDebounce = true;
        }, 256);
      } else if (canRunDebounce) {
        run(elementWidth, elementHeight, contentRects, config);
      }
    });
  });
}

export { createLayout };
